/*******************************************************************************
*                                                                              *
*   ThreadComm.cpp :                                                           *
*                                                                              *
*   source code for ThreadComm, thread in charge of communications with mbs270 *
*    . ThreadComm : main function                                              *
*    . tc_init : initialization procedure                                      *
*    . tc_stop : stopping procedure                                            *
*    . tc_processMessage : Message processing function                         *
*    . tc_recvInitData : blocking read of initial data                         *
*                                                                              *
********************************************************************************
*   SUIVI DES MODIFICATIONS                                                    *
*                                                                              *
*   14/09/2006 : cration                                                      *
*   14/01/2007 : modifs protocole communication, machine  tat                *
*                                                                              *
*******************************************************************************/

#include <sys/types.h>
#include <sys/timeb.h>

#include "stdafx.h"
#include "Client_demo.h"

//#define TC_DEF_NO_COMM
//#define TC_DEF_DEBUG

#ifdef TC_DEF_DEBUG
struct _timeb ts, te, tt;
double ts_d, te_d, tt_d, dtr;
#endif

typedef struct tc_context{
	unsigned char *recvbuf;
} tc_context_t;


void tc_traceErr(char* str1, int n, char* str2)
{
	static struct _timeb	t;
	static struct tm		*pDate;

	_ftime(&t);
	pDate = gmtime(&t.time);
	printf("%02d/%02d/%02d %02d:%02d:%02d.%03d %s %4d : %s\n",
			pDate->tm_mday,
			pDate->tm_mon+1,
			pDate->tm_year-100,
			pDate->tm_hour,
			pDate->tm_min,
			pDate->tm_sec,
			t.millitm, str1, n, str2);
}



void tc_conv_yuv_to_bgr(double y, double u, double v, unsigned char *dest)
{
	double r, g, b;
	r = y + 1.403 * v; if (r < 0.0) r = 0.0; if (r > 255.0) r = 255.0;
	g = y - 0.344 * u - 0.714 * v; if (g < 0.0) g = 0.0; if (g > 255.0) g = 255.0;
	b = y + 1.770 * u; if (b < 0.0) b = 0.0; if (b > 255.0) b = 255.0;
	*dest = (unsigned char) b;
	*(dest+1) = (unsigned char) g;
	*(dest+2) = (unsigned char) r;
}


// tc_recvSocket : blocking read on socket
// return code: 0=OK, !0=error
int tc_recvSocket(char* pBuf, int SzBuf)
{
#ifdef TC_DEF_NO_COMM
	gCtx.InitData.sensor = MD_SENSOR_OV7720;
	gCtx.InitData.formats = MD_FMT_YUV422P|MD_FMT_YUYV|MD_FMT_GRB422P|MD_FMT_GRGB|
		MD_FMT_BAYER8|MD_FMT_Y8|MD_FMT_HGRAD|MD_FMT_VGRAD|MD_FMT_EDGE;
	gCtx.InitData.default_fmt = MD_FMT_YUV422P;
	gCtx.InitData.resolutions = MD_RES_QVGA|MD_RES_VGA|MD_RES_QWVGA|MD_RES_WVGA;
	gCtx.InitData.default_res = MD_RES_VGA;

	SuspendThread(gCtx.hThreadComm);
	return 0;
#else
	int rc, nboct, nb_not_received;
	char *ptr, bDcnx;
	
	rc = 0;
	nb_not_received = SzBuf;
	ptr = pBuf;
	bDcnx = FALSE;

	while (nb_not_received > 0)
	{
		nboct = recv(gCtx.Sock, ptr, nb_not_received, 0);
		if (nboct == SOCKET_ERROR)
		{
			// Erreur
			tc_traceErr("tc_recvSocket", __LINE__, "Error recv");
			printf("EC = %d\n",WSAGetLastError());
			rc = -1;
			break;
		}
		else if (nboct == 0)
		{
			// mbs270 is disconnected
			tc_traceErr("tc_recvSocket", __LINE__, "mbs270 disconnected");
			rc = -1;
			break;
		}
		else
		{
			nb_not_received = nb_not_received - nboct;
			ptr = ptr + nboct;
		}
	}

	return rc;
#endif
}


// sendSocket : blocking write to mbs270
// return code: none, if write fail, application is stopped
void sendSocket(char* pBuf, int szBuf)
{
	printf("sendSocket %d, szBuf = %d\n", *((int*)pBuf), szBuf);
#ifdef TC_DEF_NO_COMM
	return;
#else
	int rc, nboct, nb_not_sent;
	char* ptr;

	rc = 1;
	nb_not_sent = szBuf;
	ptr = pBuf;

	while (nb_not_sent > 0)
	{
		nboct = send(gCtx.Sock, ptr, nb_not_sent, 0);
		if (nboct == SOCKET_ERROR)
		{
			tc_traceErr("sendSocket", __LINE__, "Error send");
			printf("EC = %d\n",WSAGetLastError());
			rc = 0;
			break;	// stop sending
		}
		else
		{
			nb_not_sent = nb_not_sent - nboct;
			ptr = ptr + nboct;
		}
	}

	if (rc == 0)
	{
		// stop client
		PostQuitMessage(0);
	}
#endif
}


// tc_init : initialization procedure
// return code: 0=OK, !0=error
int tc_init(tc_context_t *pCtx)
{

	int rc = 0;

	pCtx->recvbuf = NULL;

#ifdef TC_DEF_NO_COMM
	if (1) {
		if (1) {
#else
	// TCP/IP socket creation
	gCtx.Sock = socket(AF_INET, SOCK_STREAM, 0);
	if (gCtx.Sock == INVALID_SOCKET)
	{
		tc_traceErr("tc_init", __LINE__, "Error socket");
		printf(" EC = %d", WSAGetLastError());
		rc = -1;
	}
	else
	{
		SOCKADDR_IN sin;
		sin.sin_addr.s_addr = inet_addr(gCtx.IPaddr);
		sin.sin_family	    = AF_INET;
		sin.sin_port	    = htons(MD_SOCKET_PORT);

		// Connexion attempt
		if (connect(gCtx.Sock, (SOCKADDR *)&sin, sizeof(sin)) == 0)
		{
#endif
			tc_traceErr("tc_init", __LINE__, "Connexion avec mbs270 tablie");
		}
		else
		{
			tc_traceErr("tc_init", __LINE__, "Error connect");
			printf("EC = %d\n", WSAGetLastError());
			rc = -1;
		}
	}

	gCtx.init_rc = rc;

	return rc;
}


// tc_recvInitData : blocking read of initial data
// return code: 0=OK, !0=error
int tc_recvInitData()
{
	int rc;

	rc = 0;

	if (tc_recvSocket((char*)&gCtx.InitData, sizeof(gCtx.InitData)) != 0)
	{
		tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
		rc = -1;
	}
#ifdef TC_DEF_DEBUG
	else
	{
		printf("Initialization Data:\n");
		printf("sensor = %d\n", gCtx.InitData.sensor);
		printf("resolutions = 0x%08x - default_res = 0x%08x\n",
			gCtx.InitData.resolutions, gCtx.InitData.default_res);
		printf("formats = 0x%08x - default_fmt = 0x%08x\n",
			gCtx.InitData.formats, gCtx.InitData.default_fmt);
	}
#endif

	return rc;
}

		
// tc_processMessage : message processing function
// return code: 0=OK, !0=error
int tc_processMessage(tc_context_t *pCtx, md_msghdr_t *pHdr)
{
	unsigned char *image, *puc;
	unsigned int compute_time_us;
	int ii;
	int rc = 0;
	char str2display[100];
	mdc_mode_attr_t *pModeAttr = &gCtx.ModeAttr[gCtx.ModeIdx];

	if (pCtx->recvbuf == NULL) {
		if ((gCtx.MaxWidth) && (gCtx.MaxHeight)) {
			pCtx->recvbuf = (unsigned char*) malloc(gCtx.MaxWidth * gCtx.MaxHeight * 3);
			if (pCtx->recvbuf == NULL) {
				perror("malloc pCtx->recvbuf");
				return -1;
			}
		}
	}
	image = pCtx->recvbuf;

	switch (pHdr->type)
	{
	case MD_MSG_INITDATA :
#ifdef TC_DEF_DEBUG
printf("before tc_recvSocket initdata, len = %d\n", pHdr->bodylen);
#endif
		if (tc_recvInitData() != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvInitData");
			rc = -1;
		}
		break;

	case MD_MSG_STRING :
#ifdef TC_DEF_DEBUG
printf("before tc_recvSocket string, len = %d\n", pHdr->bodylen);
#endif
		if (tc_recvSocket(str2display, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
			SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, str2display);
		break;

	case MD_MSG_IMG_Y8 :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket Y8 image, size = %d\n", tt_d, pHdr->bodylen);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_Y8)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, pModeAttr->active_str);

				// Convert Y8 image to RGB24 grey level image
				puc = gCtx.Image;
				int y;
				for (ii=0; ii<pHdr->bodylen; ii++)
				{
					y = (int) image[ii];
					*puc = y; puc++;
					*puc = y; puc++;
					*puc = y; puc++;
				}
				// refresh image display
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_YUYV :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket YUYV image, size = %d\n", tt_d, pHdr->bodylen);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_YUYV)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, pModeAttr->active_str);

				// Convert YUYV image to RGB24 image
				// Y, U and V components are packed as Yn Un Yn+1 Vn+1
				puc = gCtx.Image;
				double y, u, v;
				for (ii=0; ii<pHdr->bodylen/4; ii++)
				{
					// TODO
					y = (double) image[ii*4  ];
					u = (double) image[ii*4+1] - 128.0;
					v = (double) image[ii*4+3] - 128.0;
					tc_conv_yuv_to_bgr(y, u, v, puc); puc += 3;

					y  = (double) image[ii*4+2];
					tc_conv_yuv_to_bgr(y, u, v, puc); puc += 3;
				}
				// refresh image display
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_YUV422P :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket YUV422P image, size = %d\n", tt_d, pHdr->bodylen);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_YUV422P)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, pModeAttr->active_str);

				// Convert planar YUV422 image to RGB24 image
				// Y, U and V components are in 3 separate planes:
				//   Y plane starts at image and is defined for every pixels
				//   U and V are defined for half pixel only; U plane starts at
				//   image + pHdr->bodylen/2 and V at image + pHdr->bodylen * 3/4.
				puc = gCtx.Image;
				unsigned char *py  = image;
				unsigned char *pu = py + pHdr->bodylen / 2;
				unsigned char *pv = pu + pHdr->bodylen / 4;
				double y, u, v;
				for (ii=0; ii<pHdr->bodylen/4; ii++)
				{
					y = (double) *py;
					u = (double) *pu - 128.0;
					v = (double) *pv - 128.0;
					tc_conv_yuv_to_bgr(y, u, v, puc); puc += 3;
					py++;
					y = (double) *py;
					tc_conv_yuv_to_bgr(y, u, v, puc); puc += 3;
					py++; pu++; pv++;
				}
				// refresh image display
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_GRB422P :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket image\n", tt_d);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_GRB422P)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, pModeAttr->active_str);
printf("r");
				// Convert raw RGB image to RGB24 image
				// R, G and B components are in 3 separate planes:
				//   G plane starts at image and is defined for every pixels
				//   R and B are defined for half pixel only; B plane starts at
				//   image + pHdr->bodylen/2 and R at image + pHdr->bodylen * 3/4.
				puc = gCtx.Image;
				unsigned char *pg = image;
				unsigned char *pr = pg + pHdr->bodylen / 2;
				unsigned char *pb = pr + pHdr->bodylen / 4;
				for (ii=0; ii<pHdr->bodylen/4; ii++)
				{
					*puc = *pb; puc++; // Bn
					*puc = *pg; puc++; // Gn
					*puc = *pr; puc++; // Rn
					pg++;
					*puc = *pb; puc++; // Bn+1
					*puc = *pg; puc++; // Gn+1
					*puc = *pr; puc++; // Rn+1
					pg++; pr++; pb++;
				}

				// refresh image display
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_GRGB :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket image\n", tt_d);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_GRGB)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, pModeAttr->active_str);
printf("r");
				// Convert GRGB image to RGB24 image
				// G, R and B components are packed as Gn Rn Gn+1 Bn+1
				puc = gCtx.Image;
				for (ii=0; ii<pHdr->bodylen/4; ii++)
				{
					*puc = image[ii*4+3]; puc++; // Bn
					*puc = image[ii*4  ]; puc++; // Gn
					*puc = image[ii*4+1]; puc++; // Rn
					*puc = image[ii*4+3]; puc++; // Bn+1
					*puc = image[ii*4+2]; puc++; // Gn+1
					*puc = image[ii*4+1]; puc++; // Rn+1
				}

				// refresh image display
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_BAYER :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket image\n", tt_d);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_BAYER8)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, pModeAttr->active_str);
printf("r");
				// Convert Bayer image to RGB24 image
				// even lines: GRGR..., odd lines: BGBG...
				int imageWidth = gCtx.ResAttr[gCtx.ResIdx].width;
				int bayerBPP = 1;
				int bayerBPL = bayerBPP * imageWidth;
				int rgb24BPP = 3;
				int rgb24BPL = rgb24BPP * imageWidth;
				unsigned char *pgr = image;
				unsigned char *pgrNext, *pbg;
				unsigned char *pend, *pendline;
				unsigned char *peven = gCtx.Image;
				unsigned char *podd;

				pend = pgr + gCtx.ResAttr[gCtx.ResIdx].height * bayerBPL;
				while (pgr < pend) {
					pbg = pgr + bayerBPL;
					pgrNext = pbg + bayerBPL;
					if (pgrNext >= pend) pgrNext = pgr;

					// Because algorithms are different for even and odd lines
					// process 2 lines simultaneously 
					podd = peven + rgb24BPL;
					pendline = pgr + bayerBPL;
					while (pgr < pendline) {
						*peven = *pbg; peven++;			// Bn even line
						*peven = *pgr; peven++;			// Gn even line
						*peven = *(pgr+1); peven++;		// Rn even line
						*peven = *pbg; peven++;			// Bn+1 even line
						*peven = *(pbg+1); peven++;		// Gn+1 even line
						*peven = *(pgr+1); peven++;		// Rn+1 even line
						*podd = *pbg; podd++;			// Bn odd line
						*podd = *pgrNext; podd++;		// Gn odd line
						*podd = *(pgrNext+1); podd++;	// Rn odd line
						*podd = *pbg; podd++;			// Bn+1 odd line
						*podd = *(pbg+1); podd++;		// Gn+1 odd line
						*podd = *(pgrNext+1); podd++;	// Rn+1 odd line
						pgr += (bayerBPP*2);
						pbg += (bayerBPP*2);
						pgrNext += (bayerBPP*2);
					}

					// process next 2 lines
					pgr += bayerBPL;
					peven += rgb24BPL;
				}

				// refresh image display
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_HGRAD :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket horizontal gradient image\n", tt_d);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_HGRAD)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				/* computation time is after gradient data */
				compute_time_us = *((int*)(image + pHdr->bodylen - sizeof(int)));
				sprintf(str2display, pModeAttr->active_str, compute_time_us);
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, str2display);

				// Convert 8bits/pixel image to RGB24 grey level image
				puc = gCtx.Image;
				for (ii=0; ii<pHdr->bodylen - sizeof(int); ii++)
				{
					*puc = image[ii]; puc++;
					*puc = image[ii]; puc++;
					*puc = image[ii]; puc++;
				}
				/* raffraichissement affichage image */
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_VGRAD :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket vertical gradient image\n", tt_d);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_VGRAD)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				/* computation time is after gradient data */
				compute_time_us = *((int*)(image + pHdr->bodylen - sizeof(int)));
				sprintf(str2display, pModeAttr->active_str, compute_time_us);
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, str2display);

				// Convert 8bits/pixel image to RGB24 grey level image
				puc = gCtx.Image;
				for (ii=0; ii<pHdr->bodylen - sizeof(int); ii++)
				{
					*puc = image[ii]; puc++;
					*puc = image[ii]; puc++;
					*puc = image[ii]; puc++;
				}
				/* raffraichissement affichage image */
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_IMG_EDGE :
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f before tc_recvSocket vertical gradient image\n", tt_d);
#endif
		if (tc_recvSocket((char*)image, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			if (pModeAttr->fmt == MD_FMT_EDGE)
			{
#ifdef TC_DEF_DEBUG
_ftime( &tt ); tt_d = (double) (tt.time) + (double) (tt.millitm) / 1000.0;
printf("%f after tc_recvSocket image\n", tt_d);
#endif
				/* computation time is after gradient data */
				compute_time_us = *((int*)(image + pHdr->bodylen - sizeof(int)));
				sprintf(str2display, pModeAttr->active_str, compute_time_us);
				SetDlgItemText(gCtx.hClient_demoDlg, IDC_STATIC_MESSAGE, str2display);

				// Compute gradient norm and display in RGB24
				puc = gCtx.Image;
				unsigned char norm;
				for (ii=0; ii<pHdr->bodylen - sizeof(int); ii+=2)
				{
					norm = (unsigned char) sqrt((double) *((unsigned short*)(image+ii)));
					*puc = norm; puc++;
					*puc = norm; puc++;
					*puc = norm; puc++;
				}
				/* raffraichissement affichage image */
				if (gCtx.hClient_demoDlg != NULL) {
					InvalidateRect(gCtx.hClient_demoDlg, NULL, FALSE);
					UpdateWindow(gCtx.hClient_demoDlg);
				}
			}
		}
		break;

	case MD_MSG_REG_VAL :
#ifdef TC_DEF_DEBUG
printf("before tc_recvSocket regval, len = %d\n", pHdr->bodylen);
#endif
		int reg_val;
		if (tc_recvSocket((char*)&reg_val, pHdr->bodylen) != 0)
		{
			tc_traceErr("tc_processMessage", __LINE__, "Error tc_recvSocket");
			rc = -1;
		}
		else
		{
			char val_str[10];
			if ((reg_val < 0) || (reg_val > 65535))
			{
				tc_traceErr("tc_processMessage", __LINE__, "Invalid register value");
				sprintf(val_str, "???");
			}
			else
			{
				sprintf(val_str, "0x%04x", reg_val);
			}
			SetDlgItemText(gCtx.hRegisterSettingDlg, IDC_EDIT_REGISTER_VALUE, val_str);
		}
		break;

	default :
		AfxMessageBox("Unknown message header");

	}	// end switch (pHdr->type)

	return rc;
}


// tc_stop : stopping procedure
void tc_stop(tc_context_t *pCtx)
{
	// close socket
	printf("tc_stop: close socket\n");
	if (gCtx.Sock != INVALID_SOCKET) closesocket(gCtx.Sock);
	gCtx.Sock = INVALID_SOCKET;

	if (pCtx->recvbuf != NULL)
		free(pCtx->recvbuf);

	// alert Dialog on communication problem
	PostQuitMessage(0);
}


// ThreadComm thread entry point
DWORD WINAPI ThreadComm( LPVOID )
{
	char bStop;
	md_msghdr_t	MsgHdr;
	tc_context_t tcCtx;
 
	printf("   >>>   ThreadComm starts...\n");

	bStop = FALSE;
	if (tc_init(&tcCtx) != 0)
	{
		tc_traceErr("ThreadComm", __LINE__, "Error initializing ThreadComm");
		bStop = TRUE;
	}

	while (bStop == FALSE)
	{
		// blocking read of message header
#ifdef TC_DEF_DEBUG
_ftime( &ts ); ts_d = (double) (ts.time) + (double) (ts.millitm) / 1000.0;
printf("%f before tc_recvSocket header\n", ts_d);
#endif
		if (tc_recvSocket((char*)&MsgHdr, sizeof(MsgHdr)) != 0)
		{
			tc_traceErr("ThreadComm", __LINE__, "Error tc_recvSocket");
			bStop = TRUE;
		}
		else
		{
#ifdef TC_DEF_DEBUG
tc_traceErr("ThreadComm", __LINE__, "Rception en-tte");
printf("Type = %d, len = %d\n", MsgHdr.type, MsgHdr.bodylen);
#endif
			/* Message processing */
			if (tc_processMessage(&tcCtx, &MsgHdr) != 0)
			{
				tc_traceErr("ThreadComm", __LINE__, "Echec appel tc_processMessage");
				bStop = TRUE;
			}
#ifdef TC_DEF_DEBUG
			else
			{
				_ftime( &te ); te_d = (double) (te.time) + (double) (te.millitm) / 1000.0;
				dtr = te_d - ts_d;
				printf("End receive %d      Duration = %f\n", MsgHdr.type, dtr);
			}
#endif
		}

	}	// end while (bStop == FALSE)

	tc_stop(&tcCtx);
	return 0;
}
