/***************************************************/
/*                                                 */
/*  COMMON DECLARATIONS FOR MBS270 DEMO SOFTWARE   */
/*                                                 */
/*  Copyright Mobisense Systems 2007               */
/*                                                 */
/***************************************************/

#ifndef _MBS270_DEMO_H_
#define _MBS270_DEMO_H_

#define MD_SOCKET_PORT	4444

typedef enum  {
	MD_SENSOR_OV7720, \
	MD_SENSOR_MT9V032M, \
	MD_SENSOR_MT9V032C
} mcd_sensor_t;

#define MD_RES_QVGA     0x01
#define MD_RES_VGA      0x02
#define MD_RES_QWVGA    0x04
#define MD_RES_WVGA     0x08

typedef int             md_res_t;
#define MD_MAX_RES	(8*sizeof(md_res_t))

#define MD_FMT_YUV422P  (1<<0)
#define MD_FMT_YUYV     (1<<1)
#define MD_FMT_GRB422P  (1<<2)
#define MD_FMT_GRGB     (1<<3)
#define MD_FMT_BAYER8   (1<<4)
#define MD_FMT_Y8       (1<<5)
#define MD_FMT_HGRAD    (1<<16)
#define MD_FMT_VGRAD    (1<<17)
#define MD_FMT_EDGE     (1<<18)

typedef int             md_fmt_t;
#define MD_MAX_FMT	(8*sizeof(md_fmt_t))


typedef enum {
	MD_MSG_INITDATA, \
	MD_MSG_STRING, \
	MD_MSG_IMG_Y8, \
	MD_MSG_IMG_YUV422P, \
	MD_MSG_IMG_YUYV, \
	MD_MSG_IMG_GRB422P, \
	MD_MSG_IMG_GRGB, \
	MD_MSG_IMG_BAYER, \
	MD_MSG_IMG_HGRAD, \
	MD_MSG_IMG_VGRAD, \
	MD_MSG_IMG_EDGE, \
	MD_MSG_REG_VAL, \
	MD_MSG_MAX
} mcd_msg_t;

typedef enum {
	MD_RQ_QUIT, \
	MD_RQ_STDBY, \
	MD_RQ_IMG_Y8, \
	MD_RQ_IMG_YUV422P, \
	MD_RQ_IMG_YUYV, \
	MD_RQ_IMG_GRB422P, \
	MD_RQ_IMG_GRGB, \
	MD_RQ_IMG_BAYER, \
	MD_RQ_IMG_HGRAD, \
	MD_RQ_IMG_VGRAD, \
	MD_RQ_IMG_EDGE, \
	MD_RQ_GET_REG_VAL, \
	MD_RQ_SET_REG_VAL, \
	MD_RQ_RES_QVGA, \
	MD_RQ_RES_VGA, \
	MD_RQ_RES_QWVGA, \
	MD_RQ_RES_WVGA, \
	MD_RQ_MAX
} mcd_rq_t;

/* header of exchanged messages */
typedef struct _md_msghdr_ {
	int type;       /* message type, MD_MSG_*** or MCD_RQ_*** */
	int bodylen;    /* length of message body */
} md_msghdr_t;

typedef struct _md_initdata_ {
	int sensor;             /* MD_SENSOR_*** */
	int resolutions;        /* bitwise OR of MD_RES_*** */
	int default_res;        /* one of MD_RES_*** */
	md_fmt_t formats;       /* bitwise OR of MD_FMT_*** */
	md_fmt_t default_fmt;   /* one of MD_FMT_*** */
} md_initdata_t;

#endif /* _MBS270_DEMO_H_ */



