/***************************************************/
/*                                                 */
/*  DECLARATIONS FOR GRADIENT COMPUTATIONS         */
/*                                                 */
/*  Copyright Mobisense Systems 2007               */
/*                                                 */
/***************************************************/

#ifndef _GRADIENT_COMPUTATIONS_H_
#define _GRADIENT_COMPUTATIONS_H_

#include "ippdefs.h"
#include "ippIP.h"


#define GRAD_KERNEL_SIZE   3
#define GRAD_NULL_VALUE  128

typedef struct {
  int imgOffset, imgStep;
  int fltStep;
  int resOffset, resStep;
  unsigned char *fltBuf, *resBuf;
  int timeOffset, bufSize;
  IppiSize flt_roiSize, sub_roiSize;
  Ipp32s kernel[GRAD_KERNEL_SIZE];
  int kernelSize;
  int kernelAnchor;
  int divider;
} mds_grad_attr_t;


#define EDGE_KERNEL_SIZE   3
#define EDGE_NULL_VALUE    0

typedef struct {
  int imghOffset, imgvOffset, imgStep;
  int flthStep, fltvStep;
  int resOffset, resStep;
  unsigned char *flthBuf, *fltvBuf;
  unsigned short *resBuf;
  int timeOffset, bufSize;
  IppiSize flth_roiSize, fltv_roiSize, sub_roiSize;
  Ipp32s kernelh[EDGE_KERNEL_SIZE], kernelv[EDGE_KERNEL_SIZE];
  int kernelhSize, kernelvSize;
  int kernelhAnchor, kernelvAnchor;
  int dividerh, dividerv;
} mds_edge_attr_t;


/* function prototypes */
/* ------------------- */
int mds_init_hgrad(int width, int height, mds_grad_attr_t *grad);
int mds_init_vgrad(int width, int height, mds_grad_attr_t *grad);
int mds_init_edge(int width, int height, mds_edge_attr_t *edge);
int mds_compute_hgrad(unsigned char *img, mds_grad_attr_t *grad);
int mds_compute_vgrad(unsigned char *img, mds_grad_attr_t *grad);
int mds_compute_edge(unsigned char *img, mds_edge_attr_t *edge);
void mds_deinit_hgrad(mds_grad_attr_t *grad);
void mds_deinit_vgrad(mds_grad_attr_t *grad);
void mds_deinit_edge(mds_edge_attr_t *edge);

#endif /* _GRADIENT_COMPUTATIONS_H_ */



