#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>

#define I2C_DEV_NAME "/dev/i2c-%d"

int main(int argc, char *argv[])
{
  int i2c_dev, port;
  unsigned int addr, reg, val;
  char dev_name[sizeof(I2C_DEV_NAME)+4];
  char buf[2], op[2];

  printf("\n\n   ***   test_i2c_rw   ***\n\n");
  printf("Read/Write test for I2C bus from user space\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 3) goto call_error;
  port = (unsigned int) atoi(argv[1]);
  if (port < 0 || port > 255) goto call_error;
  sscanf(argv[2], "%x", &addr);
  if (addr < 0 || addr > 0xff) goto call_error;
  
  sprintf(dev_name, I2C_DEV_NAME, port);
  i2c_dev = open(dev_name, O_RDWR);
  if (i2c_dev < 0) {
    printf("%s ", dev_name);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", dev_name);

  /* select device */
  if (ioctl(i2c_dev, I2C_SLAVE, (int) addr) < 0) {
    printf("select device 0x%x\n", addr);
    perror("ioctl I2C_SLAVE");
    goto close_i2c_device;
  }

  printf("Declared device at address 0x%x\n", addr);

  while (1) {
    printf("Enter register address (between 0x00 and 0xFF, out of range ends test)\n");
    scanf("%x", &reg);
    if (reg < 0 || reg > 0xff) goto close_i2c_device;
    buf[0] = reg;
    printf("Select operation for register 0x%x: r=read, w=write\n", reg);
    scanf("%s", &op);
    if (op[0] == 'r') {
      if (write(i2c_dev, buf, 1) != 1)
	perror("write");
      else {
	if (read(i2c_dev, buf, 2) != 2)
	  perror("read");
	else
	  printf("Register 0x%x = 0x%x\n", reg, buf[1]);
      }
    }
    if (op[0] == 'w') {
      printf("Enter register value (between 0x00 and 0xFF)\n");
      scanf("%x", &val);
      if (val < 0 || val > 0xff)
	printf("Out of range\n");
      else {
	buf[1] = val;
	if (write(i2c_dev, buf, 2) != 2)
	  perror("write");
	else
	  printf("Register 0x%x = 0x%x\n", reg, buf[1]);
      }
    }
  }

 close_i2c_device:
  usleep(100000); // for proper printing
  close(i2c_dev);
  exit(0);
    
 call_error:
  printf("Error. Call syntax is: test_i2c addr reg\n");
  printf("   i2c port (value between 0 and 255 after /dev/i2c-)\n");
  printf("   i2c device address (between 0x00 and 0x7F, 7bits format!)\n");
  exit(1);
}
