#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/i2c.h>
#include <linux/i2c-dev.h>

#define I2C_DEV_NAME "/dev/i2c-%d"

int main(int argc, char *argv[])
{
  int i2c_dev, port, ii;
  unsigned int addr;
  char dev_name[sizeof(I2C_DEV_NAME)+4];

  printf("\n\n   ***   test_smbus   ***\n\n");
  printf("Basic test for SMBUS from user space\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 3) goto call_error;
  port = (unsigned int) atoi(argv[1]);
  if (port < 0 || port > 255) goto call_error;
  sscanf(argv[2], "%x", &addr);
  if (addr < 0 || addr > 0xff) goto call_error;
  
  sprintf(dev_name, I2C_DEV_NAME, port);
  i2c_dev = open(dev_name, O_RDWR);
  if (i2c_dev < 0) {
    printf("%s ", dev_name);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", dev_name);

  /* select device */
  if (ioctl(i2c_dev, I2C_SLAVE, (int) addr) < 0) {
    printf("select device 0x%x ", addr);
    perror("ioctl I2C_SLAVE");
    goto close_i2c_device;
  }

  printf("Declared device at address 0x%x\n", addr);

  __s32 val;
  for (ii=0; ii<8; ii++) {
    val = i2c_smbus_read_byte(i2c_dev);
    if (val < 0) {
      perror("i2c_smbus_read_byte");
      break;
    }
    printf("%d: 0x%02x\n", ii, val);
  }

 close_i2c_device:
  usleep(100000); // for proper printing
  close(i2c_dev);
  exit(0);
    
 call_error:
  printf("Error. Call syntax is: test_smbus port addr\n");
  printf("   i2c port (value between 0 and 255 after /dev/i2c-)\n");
  printf("   i2c device address (between 0x00 and 0xFF)\n");
  exit(1);
}
