 
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <fcntl.h>
#include <errno.h>

#include "ippdefs.h"
#include "ippIP.h"


void print_buf(Ipp8u* buf, int len)
{
  int ii,jj;
  printf("srcBuf:\n");
  for (jj=0;jj<3;jj++) {
    for (ii=0;ii<len;ii++) printf("%3d  ", buf[ii + len*jj]);
    printf("\n");
  }
}

void print_kernel(Ipp32s* kernel, int divisor)
{
  printf("kernel (divisor = %d):\n", divisor);
  printf("%d\n%d\n%d\n", kernel[0], kernel[1], kernel[2]);
}

void test_filter(Ipp32s *kernel)
{
  IppStatus status;
  Ipp8u srcBuf[9] = {0,1,2,3,4,5,6,7,8};
  Ipp8u dstBuf[3];
  IppiSize roiSize = {3,1};
  int divisor = 1;

  print_buf(srcBuf,3);
  print_kernel(kernel, divisor);
  status = ippiFilterColumn_8u_C1R(srcBuf+3,
				   3,
				   dstBuf,
				   1,
				   roiSize,
				   kernel,
				   3,
				   1,
				   divisor);
  if (status != ippStsNoErr)
    printf("ippiFilter_8u_C1R ERROR, status = %d\n", status);
  else
    printf("dstBuf = %d %d %d\n\n", dstBuf[0], dstBuf[1], dstBuf[2]);
}

void test1(Ipp32s *kernel)
{
  printf("test1: shows match between pixels and kernel coefficients\n");
  memset(kernel, 0, sizeof(Ipp32s)*3); kernel[0] = 1;
  test_filter(kernel);
  memset(kernel, 0, sizeof(Ipp32s)*3); kernel[1] = 1;
  test_filter(kernel);
  memset(kernel, 0, sizeof(Ipp32s)*3); kernel[2] = 1;
  test_filter(kernel);
  memset(kernel, 0, sizeof(Ipp32s)*3); kernel[0] = -1;
  test_filter(kernel);
  printf("   !!!   does not accept negative results   !!!\n");

  printf("Press return key to continue...\n");
  fflush(stdin); getchar();
}

#define BUF_LEN 16
void test2(Ipp32s *kernel)
{
  int ii;
  IppStatus status;
  printf("test2: horizontal gradient computation with Sobel mask\n");
  kernel[0] = 1; kernel[1] = 2; kernel[2] = 1;
  Ipp8u srcBuf[3*BUF_LEN] = {0,0,0,10,10,10,0,0,0,0,0,0,0,20,20,0,
                             0,0,0,10,10,10,0,0,0,0,0,0,0,20,20,0,
                             0,0,0,10,10,10,0,0,0,0,0,0,0,20,20,0};
  Ipp8u fltBuf[BUF_LEN];
  IppiSize roiSize = {BUF_LEN,1};
  int divisor = 4;
  print_buf(srcBuf,BUF_LEN);
  print_kernel(kernel, divisor);
  status = ippiFilterColumn_8u_C1R(srcBuf+BUF_LEN,
				   BUF_LEN,
				   fltBuf,
				   BUF_LEN,
				   roiSize,
				   kernel,
				   3,
				   1,
				   divisor);
  if (status != ippStsNoErr) {
    printf("ippiFilter_8u_C1R ERROR, status = %d\n", status);
    return;
  }
  printf("fltBuf:\n");
  for (ii=0;ii<BUF_LEN;ii++) printf("%3d  ", fltBuf[ii]);
  printf("\n\n");

  /* add 128 to filter results to avoid negative result with substration */
  Ipp8u addBuf[BUF_LEN];
  status = ippiAddC_8u_C1RSfs(fltBuf, BUF_LEN, 128, addBuf, BUF_LEN, roiSize, 0);
  if (status != ippStsNoErr) {
    printf("ippiAddC_8u_C1RSfs ERROR, status = %d\n", status);
    return;
  }
  printf("addBuf:\n");
  for (ii=0;ii<BUF_LEN;ii++) printf("%3d  ", addBuf[ii]);
  printf("\n\n");

  /* difference computation */
  Ipp8u gradBuf[BUF_LEN-2];
  roiSize.width = BUF_LEN-2;
  status = ippiSub_8u_C1RSfs(fltBuf, BUF_LEN-2,
			     addBuf+2, BUF_LEN-2,
			     gradBuf, BUF_LEN-2,
			     roiSize, 0); /* scaleFactor = result right shift */
  if (status != ippStsNoErr){
    printf("ippiSub_8u_C1RSfs ERROR, status = %d\n", status);
    return;
  }
  printf("gradBuf:\n     ");
  for (ii=0;ii<BUF_LEN-2;ii++) printf("%3d  ", gradBuf[ii]);
  printf("\n\n");

  printf("Press return key to continue...\n");
  fflush(stdin); getchar();
}

int main(int argc, char** argv)
{
  Ipp32s kernel[3];

/*   test1(kernel); */

  test2(kernel);

  exit(0);
}
