#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#define CONFIG_PXA27x
#include <linux/pxa_pwm.h>
#undef CONFIG_PXA27x

#define PWM_DEV_NAME "/dev/pxa-pwm"

int main(int argc, char *argv[])
{
  int pwm_dev, ii;
  unsigned int duration;
  struct pxa_pwm_add pwm_add;

  printf("\n\n   ***   test_pwm2   ***\n\n");
  printf("Multichannel test for PXA PWM driver\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 3) goto call_error;
  pwm_add.f_hz = (unsigned int) atoi(argv[1]);
  pwm_add.duty = 0;
  duration = atoi(argv[2]);
  
  pwm_dev = open(PWM_DEV_NAME, O_RDWR);
  if (pwm_dev < 0) {
    printf("%s\n", PWM_DEV_NAME);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", PWM_DEV_NAME);

  int bcontinue = 1;
  for (ii=0; ii<NR_PXA_PWM; ii++) {
    pwm_add.nr = ii;
    if (ioctl(pwm_dev, PXAIOCPWMADD, (void*) &pwm_add)) {
      perror("ioctl PXAIOCPWMADD: ");
      bcontinue = 0;
      break;
    }
  }

  if (bcontinue) {
    struct timeval tvs, tv;
    struct pxa_pwm_chg pwm_chg;
    int duty = 0;
    int inc_duty = 32;
    
    printf("PWM outputs go from 0%% to 100%% at %uHz during %u seconds.\n",
	  pwm_add.f_hz, duration);

    gettimeofday(&tvs, NULL);
    pwm_chg.nr_chg = NR_PXA_PWM;
    while (1) {
      gettimeofday(&tv, NULL);
      if (tv.tv_sec > tvs.tv_sec + duration) break;
      usleep(100000);
      for (ii=0; ii<NR_PXA_PWM; ii++) {
	pwm_chg.pwm[ii].nr  = ii;
	pwm_chg.pwm[ii].val = duty;
      }
      if (ioctl(pwm_dev, PXAIOCPWMDUTY, (void*) &pwm_chg)) {
	perror("ioctl PXAIOCPWMDUTY: ");
	break;
      }
      duty += inc_duty;
      if (duty >= 1024) {
	duty = 1024;
	inc_duty = -inc_duty;
      }
      if (duty <= 0) {
	duty = 0;
	inc_duty = -inc_duty;
      }
    }
      
    for (ii=0; ii<NR_PXA_PWM; ii++) {
      if (ioctl(pwm_dev, PXAIOCPWMREM, (void*) &pwm_chg.pwm[ii].nr)) {
	perror("ioctl PXAIOCPWMREM: ");
	break;
      }
    }
  }

  close(pwm_dev);
  exit(0);
    
 call_error:
  printf("Error. Call syntax is: test_pwm2 frequency duration\n");
  printf("   frequency is the PWM frequency in Hertz\n");
  printf("   duration is the test duration in seconds\n");
  exit(1);
}
