#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <mntent.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <sys/time.h>
#include <fcntl.h>

#define SDCARD_MOUNT_DIR "/media/card"

int is_mounted()
{
        FILE *f;
	int ret;
        struct mntent mnt;
        struct mntent *mnte;
        char buf[512];

	ret = 0;

        if ((f = setmntent ("/etc/mtab", "r")) == NULL)
                goto out;

        while ((mnte = getmntent_r (f, &mnt, buf, sizeof(buf))) != NULL) {
                if (strcmp (SDCARD_MOUNT_DIR, mnt.mnt_dir) == 0) {
                        ret = 1;
                        goto out1;
                }
        }

 out1:
        endmntent (f);
 out:
        return ret;
}

int main(int argc, char *argv[])
{
	int ii, n, nbuf, buf_sz, exit_code;
	void *buf;
	FILE *f;
	char test_file_name[256];
	struct timeval tvs, tve, dtw, dtr;

	exit_code = -1;
	f = NULL;
	buf = NULL;

	printf("\n\n   ***   test_sd   ***\n\n");
	printf("Speed test for microSD card access\n\n");

	if (argc != 3) 
		goto call_error;

	buf_sz = atoi(argv[1]);
	if (buf_sz <= 0)
		goto call_error;

	nbuf = atoi(argv[2]);
	if (nbuf < 0)
		goto call_error;

	buf = malloc(buf_sz);
	if (buf == NULL) {
		perror("malloc");
		goto test_end;
	}

	/* check SDcard is mounted */
	if (!is_mounted()) {
		printf("\n   !!!   SDcard not mounted   !!!\n\n");
		goto test_end;
	}

	sprintf(test_file_name, "%s/test", SDCARD_MOUNT_DIR);
	f = fopen(test_file_name, "w");
	if (f == NULL) {
		perror("fopen");
		goto test_end;
	}

	gettimeofday(&tvs, NULL);
	for (ii = 0; ii< nbuf; ii++) {
		n = fwrite(buf, buf_sz, 1, f);
		if (n != 1) {
			perror("fwrite");
			goto test_end;
		}
	}
	gettimeofday(&tve, NULL);
	timersub(&tve, &tvs, &dtw);
	printf("%d buffers of %d bytes written to %s in %ld.%06ds\n",
	       nbuf, buf_sz, test_file_name, dtw.tv_sec, dtw.tv_usec);

	fclose(f);

	f = fopen(test_file_name, "r");
	if (f == NULL) {
		perror("fopen");
		goto test_end;
	}

	gettimeofday(&tvs, NULL);
	for (ii = 0; ii< nbuf; ii++) {
		n = fread(buf, buf_sz, 1, f);
		if (n != 1) {
			perror("fread");
			goto test_end;
		}
	}
	gettimeofday(&tve, NULL);
	timersub(&tve, &tvs, &dtr);
	printf("%d buffers of %d bytes read from %s in %ld.%06ds\n",
	       nbuf, buf_sz, test_file_name, dtr.tv_sec, dtr.tv_usec);

 test_end:
	if (f != NULL) fclose(f);
	if (buf != NULL) free(buf);
	exit(exit_code);

 call_error:
	printf("Error: call syntax is\n   test_sd buf_sz nbuf\n");
	printf("           buf_sz is the size of buffer written and read\n");
	printf("           nbuf is the number of buffers written and read\n");
	exit(-1);
}
