#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <unistd.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <linux/ioctl.h>
#include <linux/pxa_timer.h>

#define TIMER_DEV_NAME "/dev/pxa-timer"

int main(int argc, char *argv[])
{
  int timer_dev, ii;
  unsigned int duration;
  struct pxa_timer_add_chout timer_add;

  printf("\n\n   ***   test_timer   ***\n\n");
  printf("Basic test for PXA TIMER driver\n\n");
  usleep(100000); /* for proper printing */

  if (argc != 4) goto call_error;
  timer_add.nr = (unsigned int) atoi(argv[1]);
  if (timer_add.nr >= NR_PXA_CHOUT) goto call_error;
  timer_add.f_hz = (unsigned int) atoi(argv[2]);
  timer_add.ratio = 1;
  duration = atoi(argv[3]);
  
  timer_dev = open(TIMER_DEV_NAME, O_RDWR);
  if (timer_dev < 0) {
    printf("%s\n", TIMER_DEV_NAME);
    perror("open failed");
    exit(1);
  }

  printf("%s open succeeded.\n", TIMER_DEV_NAME);

  if (ioctl(timer_dev, PXAIOCTMR_ADDCO, (void*) &timer_add)) {
    perror("ioctl PXAIOCTMR_ADDCO: ");
  }
  else {
    printf("TIMER%d outputs signal at %uHz during %u seconds.\n",
	  timer_add.nr, timer_add.f_hz, duration);
    sleep(duration);
    if (ioctl(timer_dev, PXAIOCTMR_REMCO, (void*) &timer_add.nr)) {
      perror("ioctl PXAIOCTMR_REMCO: ");
    }
  }

  close(timer_dev);
  exit(0);
    
 call_error:
  printf("Error. Call syntax is: test_timer TIMER_number frequency duration\n");
  printf("   TIMER_number is the TIMER output number between 0 and %d\n",
	 NR_PXA_CHOUT-1);
  printf("   frequency is the TIMER frequency in Hertz\n");
  printf("   duration is the test duration in seconds\n");
  exit(1);
}
